# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: service-mesh-probe/service-mesh.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..common import Common_pb2 as common_dot_Common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%service-mesh-probe/service-mesh.proto\x12\rskywalking.v3\x1a\x13\x63ommon/Common.proto\"\x9d\x03\n\x11ServiceMeshMetric\x12\x11\n\tstartTime\x18\x01 \x01(\x03\x12\x0f\n\x07\x65ndTime\x18\x02 \x01(\x03\x12\x19\n\x11sourceServiceName\x18\x03 \x01(\t\x12\x1d\n\x15sourceServiceInstance\x18\x04 \x01(\t\x12\x17\n\x0f\x64\x65stServiceName\x18\x05 \x01(\t\x12\x1b\n\x13\x64\x65stServiceInstance\x18\x06 \x01(\t\x12\x10\n\x08\x65ndpoint\x18\x07 \x01(\t\x12\x0f\n\x07latency\x18\x08 \x01(\x05\x12\x14\n\x0cresponseCode\x18\t \x01(\x05\x12\x0e\n\x06status\x18\n \x01(\x08\x12)\n\x08protocol\x18\x0b \x01(\x0e\x32\x17.skywalking.v3.Protocol\x12/\n\x0b\x64\x65tectPoint\x18\x0c \x01(\x0e\x32\x1a.skywalking.v3.DetectPoint\x12\x0f\n\x07tlsMode\x18\r \x01(\t\x12\x19\n\x11internalErrorCode\x18\x0e \x01(\t\x12#\n\x03tcp\x18\x0f \x01(\x0b\x32\x16.skywalking.v3.TCPInfo\"3\n\x07TCPInfo\x12\x15\n\rreceivedBytes\x18\x01 \x01(\x03\x12\x11\n\tsentBytes\x18\x02 \x01(\x03\"\x15\n\x13MeshProbeDownstream*\'\n\x08Protocol\x12\x08\n\x04HTTP\x10\x00\x12\x08\n\x04gRPC\x10\x01\x12\x07\n\x03TCP\x10\x02\x32o\n\x18ServiceMeshMetricService\x12S\n\x07\x63ollect\x12 .skywalking.v3.ServiceMeshMetric\x1a\".skywalking.v3.MeshProbeDownstream\"\x00(\x01\x42\x8d\x01\n0org.apache.skywalking.apm.network.servicemesh.v3P\x01Z7skywalking.apache.org/repo/goapi/collect/servicemesh/v3\xaa\x02\x1dSkyWalking.NetworkProtocol.V3b\x06proto3')

_PROTOCOL = DESCRIPTOR.enum_types_by_name['Protocol']
Protocol = enum_type_wrapper.EnumTypeWrapper(_PROTOCOL)
HTTP = 0
gRPC = 1
TCP = 2


_SERVICEMESHMETRIC = DESCRIPTOR.message_types_by_name['ServiceMeshMetric']
_TCPINFO = DESCRIPTOR.message_types_by_name['TCPInfo']
_MESHPROBEDOWNSTREAM = DESCRIPTOR.message_types_by_name['MeshProbeDownstream']
ServiceMeshMetric = _reflection.GeneratedProtocolMessageType('ServiceMeshMetric', (_message.Message,), {
  'DESCRIPTOR' : _SERVICEMESHMETRIC,
  '__module__' : 'service_mesh_probe.service_mesh_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.ServiceMeshMetric)
  })
_sym_db.RegisterMessage(ServiceMeshMetric)

TCPInfo = _reflection.GeneratedProtocolMessageType('TCPInfo', (_message.Message,), {
  'DESCRIPTOR' : _TCPINFO,
  '__module__' : 'service_mesh_probe.service_mesh_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.TCPInfo)
  })
_sym_db.RegisterMessage(TCPInfo)

MeshProbeDownstream = _reflection.GeneratedProtocolMessageType('MeshProbeDownstream', (_message.Message,), {
  'DESCRIPTOR' : _MESHPROBEDOWNSTREAM,
  '__module__' : 'service_mesh_probe.service_mesh_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.MeshProbeDownstream)
  })
_sym_db.RegisterMessage(MeshProbeDownstream)

_SERVICEMESHMETRICSERVICE = DESCRIPTOR.services_by_name['ServiceMeshMetricService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n0org.apache.skywalking.apm.network.servicemesh.v3P\001Z7skywalking.apache.org/repo/goapi/collect/servicemesh/v3\252\002\035SkyWalking.NetworkProtocol.V3'
  _PROTOCOL._serialized_start=569
  _PROTOCOL._serialized_end=608
  _SERVICEMESHMETRIC._serialized_start=78
  _SERVICEMESHMETRIC._serialized_end=491
  _TCPINFO._serialized_start=493
  _TCPINFO._serialized_end=544
  _MESHPROBEDOWNSTREAM._serialized_start=546
  _MESHPROBEDOWNSTREAM._serialized_end=567
  _SERVICEMESHMETRICSERVICE._serialized_start=610
  _SERVICEMESHMETRICSERVICE._serialized_end=721
# @@protoc_insertion_point(module_scope)
