# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: logging/Logging.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..common import Common_pb2 as common_dot_Common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15logging/Logging.proto\x12\rskywalking.v3\x1a\x13\x63ommon/Common.proto\"\xea\x01\n\x07LogData\x12\x11\n\ttimestamp\x18\x01 \x01(\x03\x12\x0f\n\x07service\x18\x02 \x01(\t\x12\x17\n\x0fserviceInstance\x18\x03 \x01(\t\x12\x10\n\x08\x65ndpoint\x18\x04 \x01(\t\x12(\n\x04\x62ody\x18\x05 \x01(\x0b\x32\x1a.skywalking.v3.LogDataBody\x12\x31\n\x0ctraceContext\x18\x06 \x01(\x0b\x32\x1b.skywalking.v3.TraceContext\x12$\n\x04tags\x18\x07 \x01(\x0b\x32\x16.skywalking.v3.LogTags\x12\r\n\x05layer\x18\x08 \x01(\t\"\x9e\x01\n\x0bLogDataBody\x12\x0c\n\x04type\x18\x01 \x01(\t\x12&\n\x04text\x18\x02 \x01(\x0b\x32\x16.skywalking.v3.TextLogH\x00\x12&\n\x04json\x18\x03 \x01(\x0b\x32\x16.skywalking.v3.JSONLogH\x00\x12&\n\x04yaml\x18\x04 \x01(\x0b\x32\x16.skywalking.v3.YAMLLogH\x00\x42\t\n\x07\x63ontent\"\x17\n\x07TextLog\x12\x0c\n\x04text\x18\x01 \x01(\t\"\x17\n\x07JSONLog\x12\x0c\n\x04json\x18\x01 \x01(\t\"\x17\n\x07YAMLLog\x12\x0c\n\x04yaml\x18\x01 \x01(\t\"G\n\x0cTraceContext\x12\x0f\n\x07traceId\x18\x01 \x01(\t\x12\x16\n\x0etraceSegmentId\x18\x02 \x01(\t\x12\x0e\n\x06spanId\x18\x03 \x01(\x05\":\n\x07LogTags\x12/\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32!.skywalking.v3.KeyStringValuePair2R\n\x10LogReportService\x12>\n\x07\x63ollect\x12\x16.skywalking.v3.LogData\x1a\x17.skywalking.v3.Commands\"\x00(\x01\x42\x85\x01\n,org.apache.skywalking.apm.network.logging.v3P\x01Z3skywalking.apache.org/repo/goapi/collect/logging/v3\xaa\x02\x1dSkyWalking.NetworkProtocol.V3b\x06proto3')



_LOGDATA = DESCRIPTOR.message_types_by_name['LogData']
_LOGDATABODY = DESCRIPTOR.message_types_by_name['LogDataBody']
_TEXTLOG = DESCRIPTOR.message_types_by_name['TextLog']
_JSONLOG = DESCRIPTOR.message_types_by_name['JSONLog']
_YAMLLOG = DESCRIPTOR.message_types_by_name['YAMLLog']
_TRACECONTEXT = DESCRIPTOR.message_types_by_name['TraceContext']
_LOGTAGS = DESCRIPTOR.message_types_by_name['LogTags']
LogData = _reflection.GeneratedProtocolMessageType('LogData', (_message.Message,), {
  'DESCRIPTOR' : _LOGDATA,
  '__module__' : 'logging.Logging_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.LogData)
  })
_sym_db.RegisterMessage(LogData)

LogDataBody = _reflection.GeneratedProtocolMessageType('LogDataBody', (_message.Message,), {
  'DESCRIPTOR' : _LOGDATABODY,
  '__module__' : 'logging.Logging_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.LogDataBody)
  })
_sym_db.RegisterMessage(LogDataBody)

TextLog = _reflection.GeneratedProtocolMessageType('TextLog', (_message.Message,), {
  'DESCRIPTOR' : _TEXTLOG,
  '__module__' : 'logging.Logging_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.TextLog)
  })
_sym_db.RegisterMessage(TextLog)

JSONLog = _reflection.GeneratedProtocolMessageType('JSONLog', (_message.Message,), {
  'DESCRIPTOR' : _JSONLOG,
  '__module__' : 'logging.Logging_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.JSONLog)
  })
_sym_db.RegisterMessage(JSONLog)

YAMLLog = _reflection.GeneratedProtocolMessageType('YAMLLog', (_message.Message,), {
  'DESCRIPTOR' : _YAMLLOG,
  '__module__' : 'logging.Logging_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.YAMLLog)
  })
_sym_db.RegisterMessage(YAMLLog)

TraceContext = _reflection.GeneratedProtocolMessageType('TraceContext', (_message.Message,), {
  'DESCRIPTOR' : _TRACECONTEXT,
  '__module__' : 'logging.Logging_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.TraceContext)
  })
_sym_db.RegisterMessage(TraceContext)

LogTags = _reflection.GeneratedProtocolMessageType('LogTags', (_message.Message,), {
  'DESCRIPTOR' : _LOGTAGS,
  '__module__' : 'logging.Logging_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.LogTags)
  })
_sym_db.RegisterMessage(LogTags)

_LOGREPORTSERVICE = DESCRIPTOR.services_by_name['LogReportService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n,org.apache.skywalking.apm.network.logging.v3P\001Z3skywalking.apache.org/repo/goapi/collect/logging/v3\252\002\035SkyWalking.NetworkProtocol.V3'
  _LOGDATA._serialized_start=62
  _LOGDATA._serialized_end=296
  _LOGDATABODY._serialized_start=299
  _LOGDATABODY._serialized_end=457
  _TEXTLOG._serialized_start=459
  _TEXTLOG._serialized_end=482
  _JSONLOG._serialized_start=484
  _JSONLOG._serialized_end=507
  _YAMLLOG._serialized_start=509
  _YAMLLOG._serialized_end=532
  _TRACECONTEXT._serialized_start=534
  _TRACECONTEXT._serialized_end=605
  _LOGTAGS._serialized_start=607
  _LOGTAGS._serialized_end=665
  _LOGREPORTSERVICE._serialized_start=667
  _LOGREPORTSERVICE._serialized_end=749
# @@protoc_insertion_point(module_scope)
