# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: service-mesh-probe/service-mesh-compat.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..service_mesh_probe import service_mesh_pb2 as service__mesh__probe_dot_service__mesh__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n,service-mesh-probe/service-mesh-compat.proto\x1a%service-mesh-probe/service-mesh.proto2o\n\x18ServiceMeshMetricService\x12S\n\x07\x63ollect\x12 .skywalking.v3.ServiceMeshMetric\x1a\".skywalking.v3.MeshProbeDownstream\"\x00(\x01\x42\x9e\x01\n7org.apache.skywalking.apm.network.servicemesh.v3.compatP\x01Z7skywalking.apache.org/repo/goapi/collect/servicemesh/v3\xb8\x01\x01\xaa\x02$SkyWalking.NetworkProtocol.V3.Compatb\x06proto3')



_SERVICEMESHMETRICSERVICE = DESCRIPTOR.services_by_name['ServiceMeshMetricService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n7org.apache.skywalking.apm.network.servicemesh.v3.compatP\001Z7skywalking.apache.org/repo/goapi/collect/servicemesh/v3\270\001\001\252\002$SkyWalking.NetworkProtocol.V3.Compat'
  _SERVICEMESHMETRICSERVICE._serialized_start=87
  _SERVICEMESHMETRICSERVICE._serialized_end=198
# @@protoc_insertion_point(module_scope)
