# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: profile/Profile.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..common import Common_pb2 as common_dot_Common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15profile/Profile.proto\x12\rskywalking.v3\x1a\x13\x63ommon/Common.proto\"\\\n\x17ProfileTaskCommandQuery\x12\x0f\n\x07service\x18\x01 \x01(\t\x12\x17\n\x0fserviceInstance\x18\x02 \x01(\t\x12\x17\n\x0flastCommandTime\x18\x03 \x01(\x03\"\x83\x01\n\x0eThreadSnapshot\x12\x0e\n\x06taskId\x18\x01 \x01(\t\x12\x16\n\x0etraceSegmentId\x18\x02 \x01(\t\x12\x0c\n\x04time\x18\x03 \x01(\x03\x12\x10\n\x08sequence\x18\x04 \x01(\x05\x12)\n\x05stack\x18\x05 \x01(\x0b\x32\x1a.skywalking.v3.ThreadStack\"%\n\x0bThreadStack\x12\x16\n\x0e\x63odeSignatures\x18\x01 \x03(\t\"S\n\x17ProfileTaskFinishReport\x12\x0f\n\x07service\x18\x01 \x01(\t\x12\x17\n\x0fserviceInstance\x18\x02 \x01(\t\x12\x0e\n\x06taskId\x18\x03 \x01(\t2\x90\x02\n\x0bProfileTask\x12[\n\x16getProfileTaskCommands\x12&.skywalking.v3.ProfileTaskCommandQuery\x1a\x17.skywalking.v3.Commands\"\x00\x12M\n\x0f\x63ollectSnapshot\x12\x1d.skywalking.v3.ThreadSnapshot\x1a\x17.skywalking.v3.Commands\"\x00(\x01\x12U\n\x10reportTaskFinish\x12&.skywalking.v3.ProfileTaskFinishReport\x1a\x17.skywalking.v3.Commands\"\x00\x42\x97\x01\n5org.apache.skywalking.apm.network.language.profile.v3P\x01Z<skywalking.apache.org/repo/goapi/collect/language/profile/v3\xaa\x02\x1dSkyWalking.NetworkProtocol.V3b\x06proto3')



_PROFILETASKCOMMANDQUERY = DESCRIPTOR.message_types_by_name['ProfileTaskCommandQuery']
_THREADSNAPSHOT = DESCRIPTOR.message_types_by_name['ThreadSnapshot']
_THREADSTACK = DESCRIPTOR.message_types_by_name['ThreadStack']
_PROFILETASKFINISHREPORT = DESCRIPTOR.message_types_by_name['ProfileTaskFinishReport']
ProfileTaskCommandQuery = _reflection.GeneratedProtocolMessageType('ProfileTaskCommandQuery', (_message.Message,), {
  'DESCRIPTOR' : _PROFILETASKCOMMANDQUERY,
  '__module__' : 'profile.Profile_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.ProfileTaskCommandQuery)
  })
_sym_db.RegisterMessage(ProfileTaskCommandQuery)

ThreadSnapshot = _reflection.GeneratedProtocolMessageType('ThreadSnapshot', (_message.Message,), {
  'DESCRIPTOR' : _THREADSNAPSHOT,
  '__module__' : 'profile.Profile_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.ThreadSnapshot)
  })
_sym_db.RegisterMessage(ThreadSnapshot)

ThreadStack = _reflection.GeneratedProtocolMessageType('ThreadStack', (_message.Message,), {
  'DESCRIPTOR' : _THREADSTACK,
  '__module__' : 'profile.Profile_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.ThreadStack)
  })
_sym_db.RegisterMessage(ThreadStack)

ProfileTaskFinishReport = _reflection.GeneratedProtocolMessageType('ProfileTaskFinishReport', (_message.Message,), {
  'DESCRIPTOR' : _PROFILETASKFINISHREPORT,
  '__module__' : 'profile.Profile_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.ProfileTaskFinishReport)
  })
_sym_db.RegisterMessage(ProfileTaskFinishReport)

_PROFILETASK = DESCRIPTOR.services_by_name['ProfileTask']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n5org.apache.skywalking.apm.network.language.profile.v3P\001Z<skywalking.apache.org/repo/goapi/collect/language/profile/v3\252\002\035SkyWalking.NetworkProtocol.V3'
  _PROFILETASKCOMMANDQUERY._serialized_start=61
  _PROFILETASKCOMMANDQUERY._serialized_end=153
  _THREADSNAPSHOT._serialized_start=156
  _THREADSNAPSHOT._serialized_end=287
  _THREADSTACK._serialized_start=289
  _THREADSTACK._serialized_end=326
  _PROFILETASKFINISHREPORT._serialized_start=328
  _PROFILETASKFINISHREPORT._serialized_end=411
  _PROFILETASK._serialized_start=414
  _PROFILETASK._serialized_end=686
# @@protoc_insertion_point(module_scope)
