# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: profile/ProfileCompat.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..common import Common_pb2 as common_dot_Common__pb2
from ..profile import Profile_pb2 as profile_dot_Profile__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1bprofile/ProfileCompat.proto\x1a\x13\x63ommon/Common.proto\x1a\x15profile/Profile.proto2\x90\x02\n\x0bProfileTask\x12[\n\x16getProfileTaskCommands\x12&.skywalking.v3.ProfileTaskCommandQuery\x1a\x17.skywalking.v3.Commands\"\x00\x12M\n\x0f\x63ollectSnapshot\x12\x1d.skywalking.v3.ThreadSnapshot\x1a\x17.skywalking.v3.Commands\"\x00(\x01\x12U\n\x10reportTaskFinish\x12&.skywalking.v3.ProfileTaskFinishReport\x1a\x17.skywalking.v3.Commands\"\x00\x42\xa8\x01\n<org.apache.skywalking.apm.network.language.profile.v3.compatP\x01Z<skywalking.apache.org/repo/goapi/collect/language/profile/v3\xb8\x01\x01\xaa\x02$SkyWalking.NetworkProtocol.V3.Compatb\x06proto3')



_PROFILETASK = DESCRIPTOR.services_by_name['ProfileTask']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n<org.apache.skywalking.apm.network.language.profile.v3.compatP\001Z<skywalking.apache.org/repo/goapi/collect/language/profile/v3\270\001\001\252\002$SkyWalking.NetworkProtocol.V3.Compat'
  _PROFILETASK._serialized_start=76
  _PROFILETASK._serialized_end=348
# @@protoc_insertion_point(module_scope)
