# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: management/Management.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..common import Common_pb2 as common_dot_Common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1bmanagement/Management.proto\x12\rskywalking.v3\x1a\x13\x63ommon/Common.proto\"u\n\x12InstanceProperties\x12\x0f\n\x07service\x18\x01 \x01(\t\x12\x17\n\x0fserviceInstance\x18\x02 \x01(\t\x12\x35\n\nproperties\x18\x03 \x03(\x0b\x32!.skywalking.v3.KeyStringValuePair\";\n\x0fInstancePingPkg\x12\x0f\n\x07service\x18\x01 \x01(\t\x12\x17\n\x0fserviceInstance\x18\x02 \x01(\t2\xb5\x01\n\x11ManagementService\x12X\n\x18reportInstanceProperties\x12!.skywalking.v3.InstanceProperties\x1a\x17.skywalking.v3.Commands\"\x00\x12\x46\n\tkeepAlive\x12\x1e.skywalking.v3.InstancePingPkg\x1a\x17.skywalking.v3.Commands\"\x00\x42\x8b\x01\n/org.apache.skywalking.apm.network.management.v3P\x01Z6skywalking.apache.org/repo/goapi/collect/management/v3\xaa\x02\x1dSkyWalking.NetworkProtocol.V3b\x06proto3')



_INSTANCEPROPERTIES = DESCRIPTOR.message_types_by_name['InstanceProperties']
_INSTANCEPINGPKG = DESCRIPTOR.message_types_by_name['InstancePingPkg']
InstanceProperties = _reflection.GeneratedProtocolMessageType('InstanceProperties', (_message.Message,), {
  'DESCRIPTOR' : _INSTANCEPROPERTIES,
  '__module__' : 'management.Management_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.InstanceProperties)
  })
_sym_db.RegisterMessage(InstanceProperties)

InstancePingPkg = _reflection.GeneratedProtocolMessageType('InstancePingPkg', (_message.Message,), {
  'DESCRIPTOR' : _INSTANCEPINGPKG,
  '__module__' : 'management.Management_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.InstancePingPkg)
  })
_sym_db.RegisterMessage(InstancePingPkg)

_MANAGEMENTSERVICE = DESCRIPTOR.services_by_name['ManagementService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n/org.apache.skywalking.apm.network.management.v3P\001Z6skywalking.apache.org/repo/goapi/collect/management/v3\252\002\035SkyWalking.NetworkProtocol.V3'
  _INSTANCEPROPERTIES._serialized_start=67
  _INSTANCEPROPERTIES._serialized_end=184
  _INSTANCEPINGPKG._serialized_start=186
  _INSTANCEPINGPKG._serialized_end=245
  _MANAGEMENTSERVICE._serialized_start=248
  _MANAGEMENTSERVICE._serialized_end=429
# @@protoc_insertion_point(module_scope)
