# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: language-agent/JVMMetric.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..common import Common_pb2 as common_dot_Common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1elanguage-agent/JVMMetric.proto\x12\rskywalking.v3\x1a\x13\x63ommon/Common.proto\"j\n\x13JVMMetricCollection\x12)\n\x07metrics\x18\x01 \x03(\x0b\x32\x18.skywalking.v3.JVMMetric\x12\x0f\n\x07service\x18\x02 \x01(\t\x12\x17\n\x0fserviceInstance\x18\x03 \x01(\t\"\xfb\x01\n\tJVMMetric\x12\x0c\n\x04time\x18\x01 \x01(\x03\x12\x1f\n\x03\x63pu\x18\x02 \x01(\x0b\x32\x12.skywalking.v3.CPU\x12%\n\x06memory\x18\x03 \x03(\x0b\x32\x15.skywalking.v3.Memory\x12-\n\nmemoryPool\x18\x04 \x03(\x0b\x32\x19.skywalking.v3.MemoryPool\x12\x1d\n\x02gc\x18\x05 \x03(\x0b\x32\x11.skywalking.v3.GC\x12%\n\x06thread\x18\x06 \x01(\x0b\x32\x15.skywalking.v3.Thread\x12#\n\x05\x63lazz\x18\x07 \x01(\x0b\x32\x14.skywalking.v3.Class\"T\n\x06Memory\x12\x0e\n\x06isHeap\x18\x01 \x01(\x08\x12\x0c\n\x04init\x18\x02 \x01(\x03\x12\x0b\n\x03max\x18\x03 \x01(\x03\x12\x0c\n\x04used\x18\x04 \x01(\x03\x12\x11\n\tcommitted\x18\x05 \x01(\x03\"o\n\nMemoryPool\x12%\n\x04type\x18\x01 \x01(\x0e\x32\x17.skywalking.v3.PoolType\x12\x0c\n\x04init\x18\x02 \x01(\x03\x12\x0b\n\x03max\x18\x03 \x01(\x03\x12\x0c\n\x04used\x18\x04 \x01(\x03\x12\x11\n\tcommitted\x18\x05 \x01(\x03\"H\n\x02GC\x12%\n\x05phase\x18\x01 \x01(\x0e\x32\x16.skywalking.v3.GCPhase\x12\r\n\x05\x63ount\x18\x02 \x01(\x03\x12\x0c\n\x04time\x18\x03 \x01(\x03\"\xcd\x01\n\x06Thread\x12\x11\n\tliveCount\x18\x01 \x01(\x03\x12\x13\n\x0b\x64\x61\x65monCount\x18\x02 \x01(\x03\x12\x11\n\tpeakCount\x18\x03 \x01(\x03\x12 \n\x18runnableStateThreadCount\x18\x04 \x01(\x03\x12\x1f\n\x17\x62lockedStateThreadCount\x18\x05 \x01(\x03\x12\x1f\n\x17waitingStateThreadCount\x18\x06 \x01(\x03\x12$\n\x1ctimedWaitingStateThreadCount\x18\x07 \x01(\x03\"a\n\x05\x43lass\x12\x18\n\x10loadedClassCount\x18\x01 \x01(\x03\x12\x1f\n\x17totalUnloadedClassCount\x18\x02 \x01(\x03\x12\x1d\n\x15totalLoadedClassCount\x18\x03 \x01(\x03*\x80\x01\n\x08PoolType\x12\x14\n\x10\x43ODE_CACHE_USAGE\x10\x00\x12\x10\n\x0cNEWGEN_USAGE\x10\x01\x12\x10\n\x0cOLDGEN_USAGE\x10\x02\x12\x12\n\x0eSURVIVOR_USAGE\x10\x03\x12\x11\n\rPERMGEN_USAGE\x10\x04\x12\x13\n\x0fMETASPACE_USAGE\x10\x05*\'\n\x07GCPhase\x12\x07\n\x03NEW\x10\x00\x12\x07\n\x03OLD\x10\x01\x12\n\n\x06NORMAL\x10\x02\x32\x62\n\x16JVMMetricReportService\x12H\n\x07\x63ollect\x12\".skywalking.v3.JVMMetricCollection\x1a\x17.skywalking.v3.Commands\"\x00\x42\x93\x01\n3org.apache.skywalking.apm.network.language.agent.v3P\x01Z:skywalking.apache.org/repo/goapi/collect/language/agent/v3\xaa\x02\x1dSkyWalking.NetworkProtocol.V3b\x06proto3')

_POOLTYPE = DESCRIPTOR.enum_types_by_name['PoolType']
PoolType = enum_type_wrapper.EnumTypeWrapper(_POOLTYPE)
_GCPHASE = DESCRIPTOR.enum_types_by_name['GCPhase']
GCPhase = enum_type_wrapper.EnumTypeWrapper(_GCPHASE)
CODE_CACHE_USAGE = 0
NEWGEN_USAGE = 1
OLDGEN_USAGE = 2
SURVIVOR_USAGE = 3
PERMGEN_USAGE = 4
METASPACE_USAGE = 5
NEW = 0
OLD = 1
NORMAL = 2


_JVMMETRICCOLLECTION = DESCRIPTOR.message_types_by_name['JVMMetricCollection']
_JVMMETRIC = DESCRIPTOR.message_types_by_name['JVMMetric']
_MEMORY = DESCRIPTOR.message_types_by_name['Memory']
_MEMORYPOOL = DESCRIPTOR.message_types_by_name['MemoryPool']
_GC = DESCRIPTOR.message_types_by_name['GC']
_THREAD = DESCRIPTOR.message_types_by_name['Thread']
_CLASS = DESCRIPTOR.message_types_by_name['Class']
JVMMetricCollection = _reflection.GeneratedProtocolMessageType('JVMMetricCollection', (_message.Message,), {
  'DESCRIPTOR' : _JVMMETRICCOLLECTION,
  '__module__' : 'language_agent.JVMMetric_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.JVMMetricCollection)
  })
_sym_db.RegisterMessage(JVMMetricCollection)

JVMMetric = _reflection.GeneratedProtocolMessageType('JVMMetric', (_message.Message,), {
  'DESCRIPTOR' : _JVMMETRIC,
  '__module__' : 'language_agent.JVMMetric_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.JVMMetric)
  })
_sym_db.RegisterMessage(JVMMetric)

Memory = _reflection.GeneratedProtocolMessageType('Memory', (_message.Message,), {
  'DESCRIPTOR' : _MEMORY,
  '__module__' : 'language_agent.JVMMetric_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.Memory)
  })
_sym_db.RegisterMessage(Memory)

MemoryPool = _reflection.GeneratedProtocolMessageType('MemoryPool', (_message.Message,), {
  'DESCRIPTOR' : _MEMORYPOOL,
  '__module__' : 'language_agent.JVMMetric_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.MemoryPool)
  })
_sym_db.RegisterMessage(MemoryPool)

GC = _reflection.GeneratedProtocolMessageType('GC', (_message.Message,), {
  'DESCRIPTOR' : _GC,
  '__module__' : 'language_agent.JVMMetric_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.GC)
  })
_sym_db.RegisterMessage(GC)

Thread = _reflection.GeneratedProtocolMessageType('Thread', (_message.Message,), {
  'DESCRIPTOR' : _THREAD,
  '__module__' : 'language_agent.JVMMetric_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.Thread)
  })
_sym_db.RegisterMessage(Thread)

Class = _reflection.GeneratedProtocolMessageType('Class', (_message.Message,), {
  'DESCRIPTOR' : _CLASS,
  '__module__' : 'language_agent.JVMMetric_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.Class)
  })
_sym_db.RegisterMessage(Class)

_JVMMETRICREPORTSERVICE = DESCRIPTOR.services_by_name['JVMMetricReportService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n3org.apache.skywalking.apm.network.language.agent.v3P\001Z:skywalking.apache.org/repo/goapi/collect/language/agent/v3\252\002\035SkyWalking.NetworkProtocol.V3'
  _POOLTYPE._serialized_start=1013
  _POOLTYPE._serialized_end=1141
  _GCPHASE._serialized_start=1143
  _GCPHASE._serialized_end=1182
  _JVMMETRICCOLLECTION._serialized_start=70
  _JVMMETRICCOLLECTION._serialized_end=176
  _JVMMETRIC._serialized_start=179
  _JVMMETRIC._serialized_end=430
  _MEMORY._serialized_start=432
  _MEMORY._serialized_end=516
  _MEMORYPOOL._serialized_start=518
  _MEMORYPOOL._serialized_end=629
  _GC._serialized_start=631
  _GC._serialized_end=703
  _THREAD._serialized_start=706
  _THREAD._serialized_end=911
  _CLASS._serialized_start=913
  _CLASS._serialized_end=1010
  _JVMMETRICREPORTSERVICE._serialized_start=1184
  _JVMMETRICREPORTSERVICE._serialized_end=1282
# @@protoc_insertion_point(module_scope)
