# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: language-agent/Meter.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..common import Common_pb2 as common_dot_Common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1alanguage-agent/Meter.proto\x12\rskywalking.v3\x1a\x13\x63ommon/Common.proto\"$\n\x05Label\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"M\n\x10MeterBucketValue\x12\x0e\n\x06\x62ucket\x18\x01 \x01(\x01\x12\r\n\x05\x63ount\x18\x02 \x01(\x03\x12\x1a\n\x12isNegativeInfinity\x18\x03 \x01(\x08\"U\n\x10MeterSingleValue\x12\x0c\n\x04name\x18\x01 \x01(\t\x12$\n\x06labels\x18\x02 \x03(\x0b\x32\x14.skywalking.v3.Label\x12\r\n\x05value\x18\x03 \x01(\x01\"u\n\x0eMeterHistogram\x12\x0c\n\x04name\x18\x01 \x01(\t\x12$\n\x06labels\x18\x02 \x03(\x0b\x32\x14.skywalking.v3.Label\x12/\n\x06values\x18\x03 \x03(\x0b\x32\x1f.skywalking.v3.MeterBucketValue\"\xbe\x01\n\tMeterData\x12\x36\n\x0bsingleValue\x18\x01 \x01(\x0b\x32\x1f.skywalking.v3.MeterSingleValueH\x00\x12\x32\n\thistogram\x18\x02 \x01(\x0b\x32\x1d.skywalking.v3.MeterHistogramH\x00\x12\x0f\n\x07service\x18\x03 \x01(\t\x12\x17\n\x0fserviceInstance\x18\x04 \x01(\t\x12\x11\n\ttimestamp\x18\x05 \x01(\x03\x42\x08\n\x06metric\"B\n\x13MeterDataCollection\x12+\n\tmeterData\x18\x01 \x03(\x0b\x32\x18.skywalking.v3.MeterData2\xa7\x01\n\x12MeterReportService\x12@\n\x07\x63ollect\x12\x18.skywalking.v3.MeterData\x1a\x17.skywalking.v3.Commands\"\x00(\x01\x12O\n\x0c\x63ollectBatch\x12\".skywalking.v3.MeterDataCollection\x1a\x17.skywalking.v3.Commands\"\x00(\x01\x42s\n3org.apache.skywalking.apm.network.language.agent.v3P\x01Z:skywalking.apache.org/repo/goapi/collect/language/agent/v3b\x06proto3')



_LABEL = DESCRIPTOR.message_types_by_name['Label']
_METERBUCKETVALUE = DESCRIPTOR.message_types_by_name['MeterBucketValue']
_METERSINGLEVALUE = DESCRIPTOR.message_types_by_name['MeterSingleValue']
_METERHISTOGRAM = DESCRIPTOR.message_types_by_name['MeterHistogram']
_METERDATA = DESCRIPTOR.message_types_by_name['MeterData']
_METERDATACOLLECTION = DESCRIPTOR.message_types_by_name['MeterDataCollection']
Label = _reflection.GeneratedProtocolMessageType('Label', (_message.Message,), {
  'DESCRIPTOR' : _LABEL,
  '__module__' : 'language_agent.Meter_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.Label)
  })
_sym_db.RegisterMessage(Label)

MeterBucketValue = _reflection.GeneratedProtocolMessageType('MeterBucketValue', (_message.Message,), {
  'DESCRIPTOR' : _METERBUCKETVALUE,
  '__module__' : 'language_agent.Meter_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.MeterBucketValue)
  })
_sym_db.RegisterMessage(MeterBucketValue)

MeterSingleValue = _reflection.GeneratedProtocolMessageType('MeterSingleValue', (_message.Message,), {
  'DESCRIPTOR' : _METERSINGLEVALUE,
  '__module__' : 'language_agent.Meter_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.MeterSingleValue)
  })
_sym_db.RegisterMessage(MeterSingleValue)

MeterHistogram = _reflection.GeneratedProtocolMessageType('MeterHistogram', (_message.Message,), {
  'DESCRIPTOR' : _METERHISTOGRAM,
  '__module__' : 'language_agent.Meter_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.MeterHistogram)
  })
_sym_db.RegisterMessage(MeterHistogram)

MeterData = _reflection.GeneratedProtocolMessageType('MeterData', (_message.Message,), {
  'DESCRIPTOR' : _METERDATA,
  '__module__' : 'language_agent.Meter_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.MeterData)
  })
_sym_db.RegisterMessage(MeterData)

MeterDataCollection = _reflection.GeneratedProtocolMessageType('MeterDataCollection', (_message.Message,), {
  'DESCRIPTOR' : _METERDATACOLLECTION,
  '__module__' : 'language_agent.Meter_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.MeterDataCollection)
  })
_sym_db.RegisterMessage(MeterDataCollection)

_METERREPORTSERVICE = DESCRIPTOR.services_by_name['MeterReportService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n3org.apache.skywalking.apm.network.language.agent.v3P\001Z:skywalking.apache.org/repo/goapi/collect/language/agent/v3'
  _LABEL._serialized_start=66
  _LABEL._serialized_end=102
  _METERBUCKETVALUE._serialized_start=104
  _METERBUCKETVALUE._serialized_end=181
  _METERSINGLEVALUE._serialized_start=183
  _METERSINGLEVALUE._serialized_end=268
  _METERHISTOGRAM._serialized_start=270
  _METERHISTOGRAM._serialized_end=387
  _METERDATA._serialized_start=390
  _METERDATA._serialized_end=580
  _METERDATACOLLECTION._serialized_start=582
  _METERDATACOLLECTION._serialized_end=648
  _METERREPORTSERVICE._serialized_start=651
  _METERREPORTSERVICE._serialized_end=818
# @@protoc_insertion_point(module_scope)
