# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: language-agent/Tracing.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..common import Common_pb2 as common_dot_Common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1clanguage-agent/Tracing.proto\x12\rskywalking.v3\x1a\x13\x63ommon/Common.proto\"\xa3\x01\n\rSegmentObject\x12\x0f\n\x07traceId\x18\x01 \x01(\t\x12\x16\n\x0etraceSegmentId\x18\x02 \x01(\t\x12(\n\x05spans\x18\x03 \x03(\x0b\x32\x19.skywalking.v3.SpanObject\x12\x0f\n\x07service\x18\x04 \x01(\t\x12\x17\n\x0fserviceInstance\x18\x05 \x01(\t\x12\x15\n\risSizeLimited\x18\x06 \x01(\x08\"\xf0\x01\n\x10SegmentReference\x12\'\n\x07refType\x18\x01 \x01(\x0e\x32\x16.skywalking.v3.RefType\x12\x0f\n\x07traceId\x18\x02 \x01(\t\x12\x1c\n\x14parentTraceSegmentId\x18\x03 \x01(\t\x12\x14\n\x0cparentSpanId\x18\x04 \x01(\x05\x12\x15\n\rparentService\x18\x05 \x01(\t\x12\x1d\n\x15parentServiceInstance\x18\x06 \x01(\t\x12\x16\n\x0eparentEndpoint\x18\x07 \x01(\t\x12 \n\x18networkAddressUsedAtPeer\x18\x08 \x01(\t\"\x91\x03\n\nSpanObject\x12\x0e\n\x06spanId\x18\x01 \x01(\x05\x12\x14\n\x0cparentSpanId\x18\x02 \x01(\x05\x12\x11\n\tstartTime\x18\x03 \x01(\x03\x12\x0f\n\x07\x65ndTime\x18\x04 \x01(\x03\x12-\n\x04refs\x18\x05 \x03(\x0b\x32\x1f.skywalking.v3.SegmentReference\x12\x15\n\roperationName\x18\x06 \x01(\t\x12\x0c\n\x04peer\x18\x07 \x01(\t\x12)\n\x08spanType\x18\x08 \x01(\x0e\x32\x17.skywalking.v3.SpanType\x12+\n\tspanLayer\x18\t \x01(\x0e\x32\x18.skywalking.v3.SpanLayer\x12\x13\n\x0b\x63omponentId\x18\n \x01(\x05\x12\x0f\n\x07isError\x18\x0b \x01(\x08\x12/\n\x04tags\x18\x0c \x03(\x0b\x32!.skywalking.v3.KeyStringValuePair\x12 \n\x04logs\x18\r \x03(\x0b\x32\x12.skywalking.v3.Log\x12\x14\n\x0cskipAnalysis\x18\x0e \x01(\x08\"D\n\x03Log\x12\x0c\n\x04time\x18\x01 \x01(\x03\x12/\n\x04\x64\x61ta\x18\x02 \x03(\x0b\x32!.skywalking.v3.KeyStringValuePair\"\x10\n\x02ID\x12\n\n\x02id\x18\x01 \x03(\t\"C\n\x11SegmentCollection\x12.\n\x08segments\x18\x01 \x03(\x0b\x32\x1c.skywalking.v3.SegmentObject**\n\x08SpanType\x12\t\n\x05\x45ntry\x10\x00\x12\x08\n\x04\x45xit\x10\x01\x12\t\n\x05Local\x10\x02*,\n\x07RefType\x12\x10\n\x0c\x43rossProcess\x10\x00\x12\x0f\n\x0b\x43rossThread\x10\x01*_\n\tSpanLayer\x12\x0b\n\x07Unknown\x10\x00\x12\x0c\n\x08\x44\x61tabase\x10\x01\x12\x10\n\x0cRPCFramework\x10\x02\x12\x08\n\x04Http\x10\x03\x12\x06\n\x02MQ\x10\x04\x12\t\n\x05\x43\x61\x63he\x10\x05\x12\x08\n\x04\x46\x41\x41S\x10\x06\x32\xaf\x01\n\x19TraceSegmentReportService\x12\x44\n\x07\x63ollect\x12\x1c.skywalking.v3.SegmentObject\x1a\x17.skywalking.v3.Commands\"\x00(\x01\x12L\n\rcollectInSync\x12 .skywalking.v3.SegmentCollection\x1a\x17.skywalking.v3.Commands\"\x00\x42\x93\x01\n3org.apache.skywalking.apm.network.language.agent.v3P\x01Z:skywalking.apache.org/repo/goapi/collect/language/agent/v3\xaa\x02\x1dSkyWalking.NetworkProtocol.V3b\x06proto3')

_SPANTYPE = DESCRIPTOR.enum_types_by_name['SpanType']
SpanType = enum_type_wrapper.EnumTypeWrapper(_SPANTYPE)
_REFTYPE = DESCRIPTOR.enum_types_by_name['RefType']
RefType = enum_type_wrapper.EnumTypeWrapper(_REFTYPE)
_SPANLAYER = DESCRIPTOR.enum_types_by_name['SpanLayer']
SpanLayer = enum_type_wrapper.EnumTypeWrapper(_SPANLAYER)
Entry = 0
Exit = 1
Local = 2
CrossProcess = 0
CrossThread = 1
Unknown = 0
Database = 1
RPCFramework = 2
Http = 3
MQ = 4
Cache = 5
FAAS = 6


_SEGMENTOBJECT = DESCRIPTOR.message_types_by_name['SegmentObject']
_SEGMENTREFERENCE = DESCRIPTOR.message_types_by_name['SegmentReference']
_SPANOBJECT = DESCRIPTOR.message_types_by_name['SpanObject']
_LOG = DESCRIPTOR.message_types_by_name['Log']
_ID = DESCRIPTOR.message_types_by_name['ID']
_SEGMENTCOLLECTION = DESCRIPTOR.message_types_by_name['SegmentCollection']
SegmentObject = _reflection.GeneratedProtocolMessageType('SegmentObject', (_message.Message,), {
  'DESCRIPTOR' : _SEGMENTOBJECT,
  '__module__' : 'language_agent.Tracing_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.SegmentObject)
  })
_sym_db.RegisterMessage(SegmentObject)

SegmentReference = _reflection.GeneratedProtocolMessageType('SegmentReference', (_message.Message,), {
  'DESCRIPTOR' : _SEGMENTREFERENCE,
  '__module__' : 'language_agent.Tracing_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.SegmentReference)
  })
_sym_db.RegisterMessage(SegmentReference)

SpanObject = _reflection.GeneratedProtocolMessageType('SpanObject', (_message.Message,), {
  'DESCRIPTOR' : _SPANOBJECT,
  '__module__' : 'language_agent.Tracing_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.SpanObject)
  })
_sym_db.RegisterMessage(SpanObject)

Log = _reflection.GeneratedProtocolMessageType('Log', (_message.Message,), {
  'DESCRIPTOR' : _LOG,
  '__module__' : 'language_agent.Tracing_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.Log)
  })
_sym_db.RegisterMessage(Log)

ID = _reflection.GeneratedProtocolMessageType('ID', (_message.Message,), {
  'DESCRIPTOR' : _ID,
  '__module__' : 'language_agent.Tracing_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.ID)
  })
_sym_db.RegisterMessage(ID)

SegmentCollection = _reflection.GeneratedProtocolMessageType('SegmentCollection', (_message.Message,), {
  'DESCRIPTOR' : _SEGMENTCOLLECTION,
  '__module__' : 'language_agent.Tracing_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.SegmentCollection)
  })
_sym_db.RegisterMessage(SegmentCollection)

_TRACESEGMENTREPORTSERVICE = DESCRIPTOR.services_by_name['TraceSegmentReportService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n3org.apache.skywalking.apm.network.language.agent.v3P\001Z:skywalking.apache.org/repo/goapi/collect/language/agent/v3\252\002\035SkyWalking.NetworkProtocol.V3'
  _SPANTYPE._serialized_start=1038
  _SPANTYPE._serialized_end=1080
  _REFTYPE._serialized_start=1082
  _REFTYPE._serialized_end=1126
  _SPANLAYER._serialized_start=1128
  _SPANLAYER._serialized_end=1223
  _SEGMENTOBJECT._serialized_start=69
  _SEGMENTOBJECT._serialized_end=232
  _SEGMENTREFERENCE._serialized_start=235
  _SEGMENTREFERENCE._serialized_end=475
  _SPANOBJECT._serialized_start=478
  _SPANOBJECT._serialized_end=879
  _LOG._serialized_start=881
  _LOG._serialized_end=949
  _ID._serialized_start=951
  _ID._serialized_end=967
  _SEGMENTCOLLECTION._serialized_start=969
  _SEGMENTCOLLECTION._serialized_end=1036
  _TRACESEGMENTREPORTSERVICE._serialized_start=1226
  _TRACESEGMENTREPORTSERVICE._serialized_end=1401
# @@protoc_insertion_point(module_scope)
