# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: language-agent/JVMMetricCompat.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..common import Common_pb2 as common_dot_Common__pb2
from ..language_agent import JVMMetric_pb2 as language__agent_dot_JVMMetric__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$language-agent/JVMMetricCompat.proto\x1a\x13\x63ommon/Common.proto\x1a\x1elanguage-agent/JVMMetric.proto2b\n\x16JVMMetricReportService\x12H\n\x07\x63ollect\x12\".skywalking.v3.JVMMetricCollection\x1a\x17.skywalking.v3.Commands\"\x00\x42\xa4\x01\n:org.apache.skywalking.apm.network.language.agent.v3.compatP\x01Z:skywalking.apache.org/repo/goapi/collect/language/agent/v3\xb8\x01\x01\xaa\x02$SkyWalking.NetworkProtocol.V3.Compatb\x06proto3')



_JVMMETRICREPORTSERVICE = DESCRIPTOR.services_by_name['JVMMetricReportService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n:org.apache.skywalking.apm.network.language.agent.v3.compatP\001Z:skywalking.apache.org/repo/goapi/collect/language/agent/v3\270\001\001\252\002$SkyWalking.NetworkProtocol.V3.Compat'
  _JVMMETRICREPORTSERVICE._serialized_start=93
  _JVMMETRICREPORTSERVICE._serialized_end=191
# @@protoc_insertion_point(module_scope)
