# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: event/Event.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..common import Common_pb2 as common_dot_Common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11\x65vent/Event.proto\x12\rskywalking.v3\x1a\x13\x63ommon/Common.proto\"\x8f\x02\n\x05\x45vent\x12\x0c\n\x04uuid\x18\x01 \x01(\t\x12%\n\x06source\x18\x02 \x01(\x0b\x32\x15.skywalking.v3.Source\x12\x0c\n\x04name\x18\x03 \x01(\t\x12!\n\x04type\x18\x04 \x01(\x0e\x32\x13.skywalking.v3.Type\x12\x0f\n\x07message\x18\x05 \x01(\t\x12\x38\n\nparameters\x18\x06 \x03(\x0b\x32$.skywalking.v3.Event.ParametersEntry\x12\x11\n\tstartTime\x18\x07 \x01(\x03\x12\x0f\n\x07\x65ndTime\x18\x08 \x01(\x03\x1a\x31\n\x0fParametersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"D\n\x06Source\x12\x0f\n\x07service\x18\x01 \x01(\t\x12\x17\n\x0fserviceInstance\x18\x02 \x01(\t\x12\x10\n\x08\x65ndpoint\x18\x03 \x01(\t*\x1d\n\x04Type\x12\n\n\x06Normal\x10\x00\x12\t\n\x05\x45rror\x10\x01\x32L\n\x0c\x45ventService\x12<\n\x07\x63ollect\x12\x14.skywalking.v3.Event\x1a\x17.skywalking.v3.Commands\"\x00(\x01\x42\x81\x01\n*org.apache.skywalking.apm.network.event.v3P\x01Z1skywalking.apache.org/repo/goapi/collect/event/v3\xaa\x02\x1dSkyWalking.NetworkProtocol.V3b\x06proto3')

_TYPE = DESCRIPTOR.enum_types_by_name['Type']
Type = enum_type_wrapper.EnumTypeWrapper(_TYPE)
Normal = 0
Error = 1


_EVENT = DESCRIPTOR.message_types_by_name['Event']
_EVENT_PARAMETERSENTRY = _EVENT.nested_types_by_name['ParametersEntry']
_SOURCE = DESCRIPTOR.message_types_by_name['Source']
Event = _reflection.GeneratedProtocolMessageType('Event', (_message.Message,), {

  'ParametersEntry' : _reflection.GeneratedProtocolMessageType('ParametersEntry', (_message.Message,), {
    'DESCRIPTOR' : _EVENT_PARAMETERSENTRY,
    '__module__' : 'event.Event_pb2'
    # @@protoc_insertion_point(class_scope:skywalking.v3.Event.ParametersEntry)
    })
  ,
  'DESCRIPTOR' : _EVENT,
  '__module__' : 'event.Event_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.Event)
  })
_sym_db.RegisterMessage(Event)
_sym_db.RegisterMessage(Event.ParametersEntry)

Source = _reflection.GeneratedProtocolMessageType('Source', (_message.Message,), {
  'DESCRIPTOR' : _SOURCE,
  '__module__' : 'event.Event_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.Source)
  })
_sym_db.RegisterMessage(Source)

_EVENTSERVICE = DESCRIPTOR.services_by_name['EventService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n*org.apache.skywalking.apm.network.event.v3P\001Z1skywalking.apache.org/repo/goapi/collect/event/v3\252\002\035SkyWalking.NetworkProtocol.V3'
  _EVENT_PARAMETERSENTRY._options = None
  _EVENT_PARAMETERSENTRY._serialized_options = b'8\001'
  _TYPE._serialized_start=401
  _TYPE._serialized_end=430
  _EVENT._serialized_start=58
  _EVENT._serialized_end=329
  _EVENT_PARAMETERSENTRY._serialized_start=280
  _EVENT_PARAMETERSENTRY._serialized_end=329
  _SOURCE._serialized_start=331
  _SOURCE._serialized_end=399
  _EVENTSERVICE._serialized_start=432
  _EVENTSERVICE._serialized_end=508
# @@protoc_insertion_point(module_scope)
