#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

""" This sub-package is for the convenience of deployment and automation
A CLI for running Python scripts and programs with SkyWalking Python Agent automatically attached.
`loader/sitecustomize.py` is invoked by the Python interpreter at startup.
"""

import logging


def get_cli_logger():
    """ A logger used by sw-python CLI """
    logger = logging.getLogger('skywalking-cli')
    ch = logging.StreamHandler()
    formatter = logging.Formatter('%(name)s [%(levelname)s] %(message)s')
    ch.setFormatter(formatter)
    logger.addHandler(ch)
    logger.propagate = False

    return logger


cli_logger = get_cli_logger()
