# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: common/Common.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13\x63ommon/Common.proto\x12\rskywalking.v3\"0\n\x12KeyStringValuePair\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\x1b\n\x03\x43PU\x12\x14\n\x0cusagePercent\x18\x02 \x01(\x01\"4\n\x08\x43ommands\x12(\n\x08\x63ommands\x18\x01 \x03(\x0b\x32\x16.skywalking.v3.Command\"K\n\x07\x43ommand\x12\x0f\n\x07\x63ommand\x18\x01 \x01(\t\x12/\n\x04\x61rgs\x18\x02 \x03(\x0b\x32!.skywalking.v3.KeyStringValuePair*0\n\x0b\x44\x65tectPoint\x12\n\n\x06\x63lient\x10\x00\x12\n\n\x06server\x10\x01\x12\t\n\x05proxy\x10\x02\x42\x83\x01\n+org.apache.skywalking.apm.network.common.v3P\x01Z2skywalking.apache.org/repo/goapi/collect/common/v3\xaa\x02\x1dSkyWalking.NetworkProtocol.V3b\x06proto3')

_DETECTPOINT = DESCRIPTOR.enum_types_by_name['DetectPoint']
DetectPoint = enum_type_wrapper.EnumTypeWrapper(_DETECTPOINT)
client = 0
server = 1
proxy = 2


_KEYSTRINGVALUEPAIR = DESCRIPTOR.message_types_by_name['KeyStringValuePair']
_CPU = DESCRIPTOR.message_types_by_name['CPU']
_COMMANDS = DESCRIPTOR.message_types_by_name['Commands']
_COMMAND = DESCRIPTOR.message_types_by_name['Command']
KeyStringValuePair = _reflection.GeneratedProtocolMessageType('KeyStringValuePair', (_message.Message,), {
  'DESCRIPTOR' : _KEYSTRINGVALUEPAIR,
  '__module__' : 'common.Common_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.KeyStringValuePair)
  })
_sym_db.RegisterMessage(KeyStringValuePair)

CPU = _reflection.GeneratedProtocolMessageType('CPU', (_message.Message,), {
  'DESCRIPTOR' : _CPU,
  '__module__' : 'common.Common_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.CPU)
  })
_sym_db.RegisterMessage(CPU)

Commands = _reflection.GeneratedProtocolMessageType('Commands', (_message.Message,), {
  'DESCRIPTOR' : _COMMANDS,
  '__module__' : 'common.Common_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.Commands)
  })
_sym_db.RegisterMessage(Commands)

Command = _reflection.GeneratedProtocolMessageType('Command', (_message.Message,), {
  'DESCRIPTOR' : _COMMAND,
  '__module__' : 'common.Common_pb2'
  # @@protoc_insertion_point(class_scope:skywalking.v3.Command)
  })
_sym_db.RegisterMessage(Command)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n+org.apache.skywalking.apm.network.common.v3P\001Z2skywalking.apache.org/repo/goapi/collect/common/v3\252\002\035SkyWalking.NetworkProtocol.V3'
  _DETECTPOINT._serialized_start=248
  _DETECTPOINT._serialized_end=296
  _KEYSTRINGVALUEPAIR._serialized_start=38
  _KEYSTRINGVALUEPAIR._serialized_end=86
  _CPU._serialized_start=88
  _CPU._serialized_end=115
  _COMMANDS._serialized_start=117
  _COMMANDS._serialized_end=169
  _COMMAND._serialized_start=171
  _COMMAND._serialized_end=246
# @@protoc_insertion_point(module_scope)
